import envpool.python.xla_template
import numpy as np

num_envs = 8
env = envpool.make("Pong-v5", env_type="gym", num_envs=num_envs)
obs = env.reset()
for _ in range(5000):
    act = np.random.randint(0, 5, size=num_envs)
    obs, rew, term, trunc, info = env.step(act)
    if any(term) or any(trunc):
        print(np.logical_or(term, trunc))



import jax

from gxm.wrappers import GymnaxEnvironment, PgxEnvironment, EnvpoolEnvironment


def make(id, **kwargs):
    wrapper, id = id.split("/", 1)
    Wrapper = {
        "Gymnax": GymnaxEnvironment,
        "Pgx": PgxEnvironment,
        "Envpool": EnvpoolEnvironment,
    }[wrapper]
    return Wrapper(id, **kwargs)


if __name__ == "__main__":

    # env = make("Gymnax/CartPole-v1")
    env = make("Envpool/Pong-v5")

    @jax.jit
    def rollout(key, num_steps=1000):

        def step(env_state, key):
            key_action, key_step = jax.random.split(key)
            action = jax.random.randint(key_action, (1,), 0, env.num_actions)[0]
            env_state = env.step(key_step, env_state, action)
            jax.debug.print("{}", env_state.done)
            return env_state, None

        env_state = env.init(key)
        keys = jax.random.split(key, num_steps)
        env_state, _ = jax.lax.scan(step, env_state, keys)

        return env_state

    key = jax.random.PRNGKey(0)
    num_steps = 100
    env_state = rollout(key)



import gxm.wrappers as wrappers
from gxm.core import Environment, EnvironmentState
from gxm.registration import make

__all__ = [
    "make",
    "Environment",
    "EnvironmentState",
    "wrappers",
]



from dataclasses import dataclass
from typing import Any

import jax


@jax.tree_util.register_dataclass
@dataclass
class EnvironmentState:
    state: Any
    obs: jax.Array
    reward: float | jax.Array
    done: bool | jax.Array
    info: dict[str, Any]

    def __getitem__(self, item):
        return (self.state, self.obs, self.reward, self.done, self.info)[item]

    def __iter__(self):
        return iter((self.state, self.obs, self.reward, self.done, self.info))


class Environment:
    """Base class for environments in gxm."""

    def init(self, key: jax.Array) -> EnvironmentState:
        """Initialize the environment and return the initial state."""
        raise NotImplementedError("This method should be implemented by subclasses.")

    def step(
        self,
        key: jax.Array,
        env_state: EnvironmentState,
        action: jax.Array,
    ) -> EnvironmentState:
        """Perform a step in the environment given an action."""
        raise NotImplementedError("This method should be implemented by subclasses.")

    def reset(self, key: jax.Array, env_state: EnvironmentState) -> EnvironmentState:
        """Reset the environment to its initial state."""
        raise NotImplementedError("This method should be implemented by subclasses.")

    @property
    def num_actions(self) -> int:
        """Return the number of actions available in the environment."""
        raise NotImplementedError("This method should be implemented by subclasses.")
